//+------------------------------------------------------------------+
//|                                            TestPartialOrders.mq5 |
//|                                                          tsaktuo |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "tsaktuo"
#property link      "http://www.mql5.com"
#property version   "1.00"

#include <Trade\Trade.mqh>
#include <Files\FileTxt.mqh>

CTrade cTrade;
CFileTxt cFile;

double closing_part=0.99;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Log(string text)
  {
   Print(text);
   cFile.WriteString(text+"\r\n");
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
  {
   cTrade.SetDeviationInPoints(0);
   cTrade.SetExpertMagicNumber(0);
   cTrade.SetTypeFilling(ORDER_FILLING_AON);
   cFile.Open("Logs\TestPartialOrders.log",FILE_WRITE);
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   cFile.Close();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTick()
  {
   datetime time[1];
   static datetime last_time;

   if(PositionSelect(_Symbol))
     {
      if(PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_BUY)
        {
         if(PositionGetDouble(POSITION_PROFIT)>0.0)
           {
            //Winning trade
            if(NormalizeDouble(PositionGetDouble(POSITION_PRICE_OPEN)+((PositionGetDouble(POSITION_TP)-PositionGetDouble(POSITION_PRICE_OPEN))*closing_part),_Digits)==PositionGetDouble(POSITION_PRICE_CURRENT))
              {
               //Limit reached, close half
               ClosePosition(closing_part);
              }
           }
         else
           {
            //Loosing trade
            if(NormalizeDouble(PositionGetDouble(POSITION_PRICE_OPEN)-((PositionGetDouble(POSITION_PRICE_OPEN)-PositionGetDouble(POSITION_SL))*closing_part),_Digits)==PositionGetDouble(POSITION_PRICE_CURRENT))
              {
               //Limit reached, close half
               ClosePosition(closing_part);
              }
           }
        }
      else
        {
         //Sell order
         Log("SELL order found!");
        }
     }
   else
     {
      //No orders found
      //Check if this is new Bar
      CopyTime(_Symbol,_Period,0,1,time);
      if(time[0]==last_time)
         return;
      last_time=time[0];
      //Open position
      OpenPosition();
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void WaitTillAllOrdersAreClosed()
  {
   bool wait;
   do
     {
      wait=false;
      for(int i=0;i<OrdersTotal();i++)
        {
         if(OrderSelect(OrderGetTicket(i)))
           {
            if(OrderGetInteger(ORDER_STATE)==ORDER_STATE_PLACED || OrderGetInteger(ORDER_STATE)==ORDER_STATE_STARTED)
              {
               Log("Working order found.");
               Sleep(200);//Wait a litle bit
               wait=true;
              }
           }
        }
     }
   while(wait);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void VerifyIfNoOposit(ENUM_POSITION_TYPE positionType)
  {
//Wait till all orders are closed, because some of them can open new positions
   Log("Verifying if there are no oposit orders...");
   WaitTillAllOrdersAreClosed();
   Log("Orders are all closed.");
   if(PositionSelect(_Symbol))
     {
      //Some position found
      if(PositionGetInteger(POSITION_TYPE)==positionType)
        {
         //Oposit order found!
         Log("/!\\ Oposit order found!");
         ENUM_ORDER_TYPE ordType;
         double price=0;
         MqlTick tick;
         SymbolInfoTick(_Symbol,tick);

         switch(positionType)
           {
            case  POSITION_TYPE_BUY:
               ordType=ORDER_TYPE_SELL;
               price=tick.bid;
               break;
            default:
               ordType=ORDER_TYPE_BUY;
               price=tick.ask;
               break;
           }

         Log("Making correction...");
         while(!cTrade.PositionOpen(_Symbol,ordType,PositionGetDouble(POSITION_VOLUME),price,0,0,"Correction"))
           {
            Log(StringFormat("Failed to do correction! Error: %s",cTrade.ResultRetcodeDescription()));
            //Wait a litle bit
            Sleep(200);
            //Update price value
            SymbolInfoTick(_Symbol,tick);
            if(ordType==ORDER_TYPE_SELL)
               price=tick.bid;
            else
               price=tick.ask;
           }

         Log("Oposit order closed sucessfully.");

        }
     }
   else
     {
      Log("No positions found.");
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void ClosePosition(double lots)
  {
   if(PositionGetDouble(POSITION_VOLUME)<lots)
     {
      //Not enough volume
      return;
     }
   Log("Closing position...");
   MqlTick tick;
   SymbolInfoTick(_Symbol,tick);
   if(cTrade.PositionOpen(_Symbol,ORDER_TYPE_SELL,lots,tick.bid,0,0,"Closing half"))
     {
      //Order closed sucessfully
      Log("Position closed sucessfully!");
      VerifyIfNoOposit(POSITION_TYPE_SELL);
     }
   else
     {
      Log(StringFormat("Failed to open order! Error: %s",cTrade.ResultRetcodeDescription()));
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OpenPosition()
  {
   MqlTick tick;
   SymbolInfoTick(_Symbol,tick);
   double sl = tick.ask-100.0*_Point;
   double tp = tick.ask+100.0*_Point;
   Log("Opening order...");
//There is no open positions, let's open one
   if(cTrade.PositionOpen(_Symbol,ORDER_TYPE_BUY,1.0,tick.ask,sl,tp,"Open"))
     {
      //Order sucessfully opened.
      Log("Position opened sucessfully!");
     }
   else
     {
      Log(StringFormat("Failed to open order! Error: %s",cTrade.ResultRetcodeDescription()));
     }
  }
//+------------------------------------------------------------------+
